__global__ void CylinderTransformKernel(float *input, unsigned char *in_pixels, unsigned char *out_pixels)
{
	int i = (blockIdx.x * blockDim.x) + threadIdx.x;

	// input:
	//COP
	float xo = input[0];
	float yo = input[1];
	float zo = input[2];

	//P
	float xj = input[3];
	float yj = input[4];
	float zj = input[5];

	float R2 = WIDTH_SCALE_FACTOR * WIDTH_SCALE_FACTOR;
	float halfHeight = HEIGHT/2.0;
	float halfWidth = WIDTH/2.0;

	// For stepping in the x and y directions
	float deltaXP = 2.0 * WIDTH_SCALE_FACTOR / WIDTH;
	float deltaYP = 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT;

	// Where projection intersects the planar surface
	float xp = (-halfWidth  * 2.0 * WIDTH_SCALE_FACTOR  /WIDTH ) + (i * deltaXP);
	float yp = (-halfHeight * 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT);
	float zp = 0;

	// Simplifying common expressions
	float xj_xp = xj - xp;
	float zj_zp = zj - zp;
		  
	float xj_xp2 = xj_xp * xj_xp;
	float zj_zp2 = zj_zp * zj_zp;

	float xpzj = xp * zj;
	float xjzp = xj * zp;
		  
	// Since the value of z is the same for all values of y given the same x-value,
	// we can solve for z outside the y-loop

	// Solve z-equation
	float a = xj_xp2 + zj_zp2;
	float b = 2.0 * xj_xp * (xpzj - xjzp);
	float c = (xpzj - xjzp) * (xpzj - xjzp) - R2 * zj_zp2;

	float z = (-b - sqrt(b * b - 4 * a * c)) / (2 * a);

	// Solve for x and xc, and decide u
	float x = (xpzj - xjzp + z * xj_xp) / zj_zp;
	float xc = (x *(zo - zp) + xo * (zp - z)) / (zo - z);

	int u = halfWidth * (xc / WIDTH_SCALE_FACTOR + 1);


	float y, yc;
	float yEqPart  = sqrt(((zj - z) * (zj - z) + (x - xj) * (x - xj)) / a);
	float ycEqPart = sqrt(((zo - zp) * (zo - zp) + (xo - xc) * (xo - xc)) / ((zo - z) * (zo - z) + (xo - x) * (xo - x)));
	int v;

	for( int j = 0; j< halfHeight; j++)
	{	
		// Solve for y and yc, and decide v
		y = (yp - yj) * yEqPart + yj;
		yc = (y - yo) * ycEqPart + yo;

		v = halfHeight * (yc / HEIGHT_SCALE_FACTOR + 1);

		// Put pixel value in out_pixels
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR    ];
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 1];
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 2];

		yp += deltaYP;
	}
}

__global__ void SphereTransformKernel(float *input, unsigned char *in_pixels, unsigned char *out_pixels)
{
	int i = (blockIdx.x * blockDim.x) + threadIdx.x;

	// input:
	//COP
	float xo = input[0];
	float yo = input[1];
	float zo = input[2];

	//P
	float xj = input[3];
	float yj = input[4];
	float zj = input[5];

	float R2 = WIDTH_SCALE_FACTOR * WIDTH_SCALE_FACTOR;
	float halfHeight = HEIGHT/2.0;
	float halfWidth = WIDTH/2.0;

	// For stepping in the x and y directions
	float deltaXP = 2.0 * WIDTH_SCALE_FACTOR / WIDTH;
	float deltaYP = 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT;

	// Where projection intersects the planar surface
	float xp = (-halfWidth  * 2.0 * WIDTH_SCALE_FACTOR  /WIDTH ) + (i * deltaXP);;
	float yp = 0;
	float zp = 0;


	float xj_xp, zj_zp, yj_yp, xj_xp2, yj_yp2, zj_zp2, xj2, yj2, xp2, yp2, zj2, zp2, 
		  xpyj, xpxj, xpzj, xjzp, xjyp, ypyj, zpzj, a, b, d, z, x, y, xc, yc;
	int u,v;

	for( int j = halfHeight; j < HEIGHT; j++)
	{
		// Simplifying common expressions
		xj_xp = xj - xp;
		yj_yp = yj - yp;
		zj_zp = zj - zp;
			  
		xj_xp2 = xj_xp * xj_xp;
		yj_yp2 = yj_yp * yj_yp;
		zj_zp2 = zj_zp * zj_zp;
			  
		xj2 = xj * xj;
		xp2 = xp * xp;
		yj2 = yj * yj;
		yp2 = yp * yp;
		zj2 = zj * zj;
		zp2 = zp * zp;

		xpxj = xp * xj;
		xpyj = xp * yj;
		xjyp = xj * yp;
		xpzj = xp * zj;
		xjzp = xj * zp;
		ypyj = yp * yj;
		zpzj = zp * zj;

		// Solve z-equation
		a = xj_xp2 + yj_yp2 + zj_zp2;
		b = zj * (xp2 - xpxj - yp * yj_yp)
		    + zp * (xj2 - xpxj + yj * yj_yp);
		d = zj_zp2 * (zj2 * (R2 - xp2 - yp2) - 2 * zpzj * (R2 - xpxj - ypyj)
			- (xjyp - xpyj) * (xjyp - xpyj) + zp2 * (R2 - xj2 - yj2)
			+ R2 * (xj_xp2 + yj_yp2));
		
		z = (b - sqrt(d)) / a;

		// Solve for x and xc, and decide u
		x = (xpzj - xjzp + z * xj_xp) / zj_zp;
		xc = (x *(zo - zp) + xo * (zp - z)) / (zo - z);

		u = halfWidth * (xc / WIDTH_SCALE_FACTOR + 1);

		// Solve for y and yc, and decide v
		y = (yp - yj) * sqrt(((zj - z) * (zj - z) + (x - xj) * (x - xj)) / (zj_zp2 + xj_xp2)) + yj;
		yc = (y - yo) * sqrt(((zo - zp) * (zo - zp) + (xo - xc) * (xo - xc)) / ((zo - z) * (zo - z) + (xo - x) * (xo - x))) + yo;

		v = halfHeight * (yc / HEIGHT_SCALE_FACTOR + 1);

		// Put pixel value in out_pixels
		if(u < 0 || u >= WIDTH || v < 1 || v >= HEIGHT)
		{
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = 0x55;
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = 0x55;
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = 0x55;
		}
		else
		{
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR    ];
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 1];
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 2];
		}
		yp += deltaYP;
	}
}


__global__ void CylinderTransformKernel2(float *input, unsigned char *in_pixels, unsigned char *out_pixels)
{
	// to get 1 thread per pixel
	int t = (blockIdx.x  * blockDim.x * blockDim.y) + (threadIdx.x + blockDim.x * threadIdx.y);

	int i = t % WIDTH;
	int j = (t - i) / WIDTH;

	// input:
	//COP
	float xo = input[0];
	float yo = input[1];
	float zo = input[2];

	//P
	float xj = input[3];
	float yj = input[4];
	float zj = input[5];

	float R2 = WIDTH_SCALE_FACTOR * WIDTH_SCALE_FACTOR;
	float halfHeight = HEIGHT/2.0;
	float halfWidth = WIDTH/2.0;

	// For stepping in the x and y directions
	float deltaXP = 2.0 * WIDTH_SCALE_FACTOR / WIDTH;
	float deltaYP = 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT;

	// Where projection intersects the planar surface
	float xp = (-halfWidth  * 2.0 * WIDTH_SCALE_FACTOR  /WIDTH ) + (i * deltaXP);
	float yp = (-halfHeight * 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT) + (j * deltaYP);
	float zp = 0;

	// Simplifying common expressions
	float xj_xp = xj - xp;
	float zj_zp = zj - zp;
		  
	float xj_xp2 = xj_xp * xj_xp;
	float zj_zp2 = zj_zp * zj_zp;

	float xpzj = xp * zj;
	float xjzp = xj * zp;
		  

	// Solve z-equation
	float a = xj_xp2 + zj_zp2;
	float b = 2.0 * xj_xp * (xpzj - xjzp);
	float c = (xpzj - xjzp) * (xpzj - xjzp) - R2 * zj_zp2;

	float z = (-b - sqrt(b * b - 4 * a * c)) / (2 * a);

	// Solve for x and xc, and decide u
	float x = (xpzj - xjzp + z * xj_xp) / zj_zp;
	float xc = (x *(zo - zp) + xo * (zp - z)) / (zo - z);

	int u = halfWidth * (xc / WIDTH_SCALE_FACTOR + 1);

	// Solve for y and yc, and decide v
	float y = (yp - yj) * sqrt(((zj - z) * (zj - z) + (x - xj) * (x - xj)) / a) + yj;
	float yc = (y - yo) * sqrt(((zo - zp) * (zo - zp) + (xo - xc) * (xo - xc)) / ((zo - z) * (zo - z) + (xo - x) * (xo - x))) + yo;

	int v = halfHeight * (yc / HEIGHT_SCALE_FACTOR + 1);

	// Put pixel value in out_pixels
	out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR    ];
	out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 1];
	out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 2];

}

__global__ void SphereTransformKernel2(float *input, unsigned char *in_pixels, unsigned char *out_pixels)
{
	// to get 1 thread per pixel
	int t = (blockIdx.x  * blockDim.x * blockDim.y) + (threadIdx.x + blockDim.x * threadIdx.y);

	int i = t % WIDTH;
	int j = (t - i) / WIDTH;

	// input:
	//COP
	float xo = input[0];
	float yo = input[1];
	float zo = input[2];

	//P
	float xj = input[3];
	float yj = input[4];
	float zj = input[5];

	float R2 = WIDTH_SCALE_FACTOR * WIDTH_SCALE_FACTOR;
	float halfHeight = HEIGHT/2.0;
	float halfWidth = WIDTH/2.0;

	// For stepping in the x and y directions
	float deltaXP = 2.0 * WIDTH_SCALE_FACTOR / WIDTH;
	float deltaYP = 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT;

	// Where projection intersects the planar surface
	float xp = (-halfWidth  * 2.0 * WIDTH_SCALE_FACTOR  /WIDTH ) + (i * deltaXP);
	float yp = 0 + (j * deltaYP);
	float zp = 0;

	// Simplifying common expressions
	float xj_xp = xj - xp;
	float zj_zp = zj - zp;
	float yj_yp = yj - yp;
		  
	float xj_xp2 = xj_xp * xj_xp;
	float zj_zp2 = zj_zp * zj_zp;
	float yj_yp2 = yj_yp * yj_yp;

	float xj2 = xj * xj;
	float xp2 = xp * xp;
	float zj2 = zj * zj;
	float zp2 = zp * zp;
	float yj2 = yj * yj;
	float yp2 = yp * yp;

	float xpxj = xp * xj;
	float xpzj = xp * zj;
	float xjzp = xj * zp;
	float zpzj = zp * zj;
	float xjyp = xj * yp;
	float xpyj = xp * yj;
	float ypyj = yp * yj;

	// Solve z-equation
	float a = xj_xp2 + yj_yp2 + zj_zp2;
	float b = zj * (xp2 - xpxj - yp * yj_yp)
	          + zp * (xj2 - xpxj + yj * yj_yp);
	float d = zj_zp2 * (zj2 * (R2 - xp2 - yp2) - 2 * zpzj * (R2 - xpxj - ypyj)
		      - (xjyp - xpyj) * (xjyp - xpyj) + zp2 * (R2 - xj2 - yj2)
		      + R2 * (xj_xp2 + yj_yp2));
	
	float z = (b - sqrt(d)) / a;

	// Solve for x and xc, and decide u
	float x = (xpzj - xjzp + z * xj_xp) / zj_zp;
	float xc = (x *(zo - zp) + xo * (zp - z)) / (zo - z);

	int u = halfWidth * (xc / WIDTH_SCALE_FACTOR + 1);

	// Solve for y and yc, and decide v
	float y = (yp - yj) * sqrt(((zj - z) * (zj - z) + (x - xj) * (x - xj)) / (zj_zp2 + xj_xp2)) + yj;
	float yc = (y - yo) * sqrt(((zo - zp) * (zo - zp) + (xo - xc) * (xo - xc)) / ((zo - z) * (zo - z) + (xo - x) * (xo - x))) + yo;

	int v = halfHeight * (yc / HEIGHT_SCALE_FACTOR + 1);

	// draw on the top half
	j += halfHeight;

	// Put pixel value in out_pixels
	
	if(u < 0 || u >= WIDTH || v < 1 || v >= HEIGHT)
	{
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = 0x55;
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = 0x55;
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = 0x55;
	}
	else
	{
	
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR    ];
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 1];
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 2];
	}
}

__global__ void CombinedTransformKernel(float *input, unsigned char *in_pixels, unsigned char *out_pixels)
{
	int i = (blockIdx.x * blockDim.x) + threadIdx.x;

	// input:
	//COP
	float xo = input[0];
	float yo = input[1];
	float zo = input[2];

	//P
	float xj = input[3];
	float yj = input[4];
	float zj = input[5];

	float R2 = WIDTH_SCALE_FACTOR * WIDTH_SCALE_FACTOR;
	float halfHeight = HEIGHT/2.0;
	float halfWidth = WIDTH/2.0;

	// For stepping in the x and y directions
	float deltaXP = 2.0 * WIDTH_SCALE_FACTOR / WIDTH;
	float deltaYP = 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT;

	// Where projection intersects the planar surface
	float xp = (-halfWidth  * 2.0 * WIDTH_SCALE_FACTOR  /WIDTH ) + (i * deltaXP);
	float yp = (-halfHeight * 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT);
	float zp = 0;

	// Simplifying common expressions
	float xj_xp = xj - xp;
	float zj_zp = zj - zp;
		  
	float xj_xp2 = xj_xp * xj_xp;
	float zj_zp2 = zj_zp * zj_zp;
		  
	float xj2 = xj * xj;
	float xp2 = xp * xp;
	float zj2 = zj * zj;
	float zp2 = zp * zp;

	float xpxj = xp * xj;
	float xpzj = xp * zj;
	float xjzp = xj * zp;
	float zpzj = zp * zj;
		  
	float a, b, c, d, z;
	for( int j = 0; j< HEIGHT; j++)
	{	
		// Solve z-equation:
		if(yp<0)
		{
			a = xj_xp2 + zj_zp2;
			b = 2.0 * xj_xp * (xpzj - xjzp);
			c = (xpzj - xjzp) * (xpzj - xjzp) - R2 * zj_zp2;

			z = (-b - sqrt(b * b - 4.0 * a * c)) / (2.0 * a);
		}
		else
		{
			// Simplifying more common expressions
			float yj_yp = yj - yp;
			float yj_yp2 = yj_yp * yj_yp;

			float yj2 = yj * yj;
			float yp2 = yp * yp;

			float ypyj = yp * yj;
			float xjyp = xj * yp;
			float xpyj = xp * yj;

			a = xj_xp2 + yj_yp2 + zj_zp2;
			b = zj * (xp2 - xpxj - yp * yj_yp)
			    + zp * (xj2 - xpxj + yj * yj_yp);
			//c = zj * (xp * (-xj_xp) + yp * (yp - 3 * yj) + 2 * yj2)
			//    + zp * (xj * xj_xp + yj * (-yj_yp));
			d = zj_zp2 * (zj2 * (R2 - xp2 - yp2) - 2 * zpzj * (R2 - xpxj - ypyj)
				- (xjyp - xpyj) * (xjyp - xpyj) + zp2 * (R2 - xj2 - yj2)
				+ R2 * (xj_xp2 + yj_yp2));
			//e = zj_zp2 * (zj2 * (R2 - xp2 - yp2 + 4 * yj * (-yj_yp))
			//	- 2 * zpzj * (R2 -xpxj + ypyj - 2 * yj2) + zp2 * (R2 - xj2 - yj2)
			//	+ yj2 * (R - 2 * xj + xp) - 2 * ypyj * (R2 - 2 * xj2 + xpxj)
			//	+ R2 * (xj_xp2 + yp2) - xjyp * xjyp);

			z = (b - sqrt(d)) / a;
			//z = (c - sqrt(e)) / a;
		}

		// Solve for x and xc, and decide u
		float x = (xpzj - xjzp + z * xj_xp) / zj_zp;
		float xc = (x *(zo - zp) + xo * (zp - z)) / (zo - z);

		int u = halfWidth * (xc / WIDTH_SCALE_FACTOR + 1);

		// Solve for y and yc, and decide v
		float y = (yp - yj) * sqrt(((zj - z) * (zj - z) + (x - xj) * (x - xj)) / (zj_zp2 + xj_xp2)) + yj;
		float yc = (y - yo) * sqrt(((zo - zp) * (zo - zp) + (xo - xc) * (xo - xc)) / ((zo - z) * (zo - z) + (xo - x) * (xo - x))) + yo;

		int v = halfHeight * (yc / HEIGHT_SCALE_FACTOR + 1);


		// Put pixel value in out_pixels
		if(u < 0 || u >= WIDTH || v < 1 || v >= HEIGHT)
		{
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = 0x55;
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = 0x55;
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = 0x55;
		}
		else
		{
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR    ];
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 1];
			out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 2];
		}
		yp += deltaYP;
	}
}

__global__ void CombinedTransformKernel2(float *input, unsigned char *in_pixels, unsigned char *out_pixels)
{
	// to get 1 thread per pixel
	int t = (blockIdx.x  * blockDim.x * blockDim.y) + (threadIdx.x + blockDim.x * threadIdx.y);

	int i = t % WIDTH;
	int j = (t - i) / WIDTH;

	// input:
	//COP
	float xo = input[0];
	float yo = input[1];
	float zo = input[2];

	//P
	float xj = input[3];
	float yj = input[4];
	float zj = input[5];

	float R2 = WIDTH_SCALE_FACTOR * WIDTH_SCALE_FACTOR;
	float halfHeight = HEIGHT/2.0;
	float halfWidth = WIDTH/2.0;

	// For stepping in the x and y directions
	float deltaXP = 2.0 * WIDTH_SCALE_FACTOR / WIDTH;
	float deltaYP = 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT;

	// Where projection intersects the planar surface
	float xp = (-halfWidth  * 2.0 * WIDTH_SCALE_FACTOR  /WIDTH ) + (i * deltaXP);
	float yp = (-halfHeight * 2.0 * HEIGHT_SCALE_FACTOR /HEIGHT) + (j * deltaYP);
	float zp = 0;

	// Simplifying common expressions
	float xj_xp = xj - xp;
	float zj_zp = zj - zp;
		  
	float xj_xp2 = xj_xp * xj_xp;
	float zj_zp2 = zj_zp * zj_zp;
		  
	float xj2 = xj * xj;
	float xp2 = xp * xp;
	float zj2 = zj * zj;
	float zp2 = zp * zp;

	float xpxj = xp * xj;
	float xpzj = xp * zj;
	float xjzp = xj * zp;
	float zpzj = zp * zj;
		  
	float a, b, c, d, z;
	
	// Solve z-equation:
	if(yp<0)
	{
		a = xj_xp2 + zj_zp2;
		b = 2.0 * xj_xp * (xpzj - xjzp);
		c = (xpzj - xjzp) * (xpzj - xjzp) - R2 * zj_zp2;

		z = (-b - sqrt(b * b - 4 * a * c)) / (2 * a);
	}
	else
	{
		// Simplifying more common expressions
		float yj_yp = yj - yp;
		float yj_yp2 = yj_yp * yj_yp;

		float yj2 = yj * yj;
		float yp2 = yp * yp;

		float ypyj = yp * yj;
		float xjyp = xj * yp;
		float xpyj = xp * yj;

		a = xj_xp2 + yj_yp2 + zj_zp2;
		b = zj * (xp2 - xpxj - yp * yj_yp)
		    + zp * (xj2 - xpxj + yj * yj_yp);
		//c = zj * (xp * (-xj_xp) + yp * (yp - 3 * yj) + 2 * yj2)
		//    + zp * (xj * xj_xp + yj * (-yj_yp));
		d = zj_zp2 * (zj2 * (R2 - xp2 - yp2) - 2 * zpzj * (R2 - xpxj - ypyj)
			- (xjyp - xpyj) * (xjyp - xpyj) + zp2 * (R2 - xj2 - yj2)
			+ R2 * (xj_xp2 + yj_yp2));
		//e = zj_zp2 * (zj2 * (R2 - xp2 - yp2 + 4 * yj * (-yj_yp))
		//	- 2 * zpzj * (R2 -xpxj + ypyj - 2 * yj2) + zp2 * (R2 - xj2 - yj2)
		//	+ yj2 * (R - 2 * xj + xp) - 2 * ypyj * (R2 - 2 * xj2 + xpxj)
		//	+ R2 * (xj_xp2 + yp2) - xjyp * xjyp);

		z = (b - sqrt(d)) / a;
		//z = (c - sqrt(e)) / a;
	}

	// Solve for x and xc, and decide u
	float x = (xpzj - xjzp + z * xj_xp) / zj_zp;
	float xc = (x *(zo - zp) + xo * (zp - z)) / (zo - z);

	int u = halfWidth * (xc / WIDTH_SCALE_FACTOR + 1);

	// Solve for y and yc, and decide v
	float y = (yp - yj) * sqrt(((zj - z) * (zj - z) + (x - xj) * (x - xj)) / (zj_zp2 + xj_xp2)) + yj;
	float yc = (y - yo) * sqrt(((zo - zp) * (zo - zp) + (xo - xc) * (xo - xc)) / ((zo - z) * (zo - z) + (xo - x) * (xo - x))) + yo;

	int v = halfHeight * (yc / HEIGHT_SCALE_FACTOR + 1);


	// Put pixel value in out_pixels
	if(u < 0 || u >= WIDTH || v < 1 || v >= HEIGHT)
	{
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = 0x55;
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = 0x55;
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = 0x55;
	}
	else
	{
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR    ] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR    ];
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 1] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 1];
		out_pixels[(i + j * WIDTH) * NUM_COLOR_FACTOR + 2] = in_pixels[(u + v * WIDTH) * NUM_COLOR_FACTOR + 2];
	}
}